@@ -17,6 +17,7 @@
 import inspect
 import sys
 import time
+from functools import update_wrapper
 
 import six
 
@@ -28,6 +29,27 @@ except AttributeError:
 
 
 if six.PY2:
+    from functools import WRAPPER_ASSIGNMENTS, WRAPPER_UPDATES
+
+    def wraps(fn):
+        """Do the same as six.wraps but only copy attributes that exist.
+
+        For example, object instances don't have __name__ attribute, so
+        six.wraps fails. This is fixed in Python 3
+        (https://bugs.python.org/issue3445), but didn't get backported to six.
+
+        Also, see https://github.com/benjaminp/six/issues/250.
+        """
+
+        def filter_hasattr(obj, attrs):
+            return tuple(a for a in attrs if hasattr(obj, a))
+
+        return six.wraps(
+            fn,
+            assigned=filter_hasattr(fn, WRAPPER_ASSIGNMENTS),
+            updated=filter_hasattr(fn, WRAPPER_UPDATES),
+        )
+
     def capture(fut, tb):
         # TODO(harlowja): delete this in future, since its
         # has to repeatedly calculate this crap.
@@ -36,7 +58,11 @@ if six.PY2:
     def getargspec(func):
         # This was deprecated in Python 3.
         return inspect.getargspec(func)
+
+
 else:
+    from functools import wraps  # noqa
+
     def capture(fut, tb):
         fut.set_exception(tb[1])
 
@@ -59,13 +85,13 @@ def find_ordinal(pos_num):
     if pos_num == 0:
         return "th"
     elif pos_num == 1:
-        return 'st'
+        return "st"
     elif pos_num == 2:
-        return 'nd'
+        return "nd"
     elif pos_num == 3:
-        return 'rd'
+        return "rd"
     elif pos_num >= 4 and pos_num <= 20:
-        return 'th'
+        return "th"
     else:
         return find_ordinal(pos_num % 10)
 
@@ -111,3 +137,23 @@ try:
     now = time.monotonic  # noqa
 except AttributeError:
     from monotonic import monotonic as now  # noqa
+
+
+class cached_property(object):
+    """A property that is computed once per instance.
+
+    Upon being computed it replaces itself with an ordinary attribute. Deleting
+    the attribute resets the property.
+
+    Source: https://github.com/bottlepy/bottle/blob/1de24157e74a6971d136550afe1b63eec5b0df2b/bottle.py#L234-L246
+    """  # noqa: E501
+
+    def __init__(self, func):
+        update_wrapper(self, func)
+        self.func = func
+
+    def __get__(self, obj, cls):
+        if obj is None:
+            return self
+        value = obj.__dict__[self.func.__name__] = self.func(obj)
+        return value
