@@ -1,4 +1,6 @@
-# Copyright 2016 Julien Danjou
+# -*- encoding: utf-8 -*-
+#
+# Copyright 2016–2021 Julien Danjou
 # Copyright 2016 Joshua Harlow
 # Copyright 2013-2014 Ray Holder
 #
@@ -15,6 +17,7 @@
 # limitations under the License.
 
 import abc
+import re
 
 import six
 
@@ -24,7 +27,7 @@ class retry_base(object):
     """Abstract base class for retry strategies."""
 
     @abc.abstractmethod
-    def __call__(self, attempt):
+    def __call__(self, retry_state):
         pass
 
     def __and__(self, other):
@@ -36,7 +39,8 @@ class retry_base(object):
 
 class _retry_never(retry_base):
     """Retry strategy that never rejects any result."""
-    def __call__(self, attempt):
+
+    def __call__(self, retry_state):
         return False
 
 
@@ -45,7 +49,8 @@ retry_never = _retry_never()
 
 class _retry_always(retry_base):
     """Retry strategy that always rejects any result."""
-    def __call__(self, attempt):
+
+    def __call__(self, retry_state):
         return True
 
 
@@ -58,9 +63,11 @@ class retry_if_exception(retry_base):
     def __init__(self, predicate):
         self.predicate = predicate
 
-    def __call__(self, attempt):
-        if attempt.failed:
-            return self.predicate(attempt.exception())
+    def __call__(self, retry_state):
+        if retry_state.outcome.failed:
+            return self.predicate(retry_state.outcome.exception())
+        else:
+            return False
 
 
 class retry_if_exception_type(retry_if_exception):
@@ -69,7 +76,8 @@ class retry_if_exception_type(retry_if_exception):
     def __init__(self, exception_types=Exception):
         self.exception_types = exception_types
         super(retry_if_exception_type, self).__init__(
-            lambda e: isinstance(e, exception_types))
+            lambda e: isinstance(e, exception_types)
+        )
 
 
 class retry_unless_exception_type(retry_if_exception):
@@ -78,13 +86,14 @@ class retry_unless_exception_type(retry_if_exception):
     def __init__(self, exception_types=Exception):
         self.exception_types = exception_types
         super(retry_unless_exception_type, self).__init__(
-            lambda e: not isinstance(e, exception_types))
+            lambda e: not isinstance(e, exception_types)
+        )
 
-    def __call__(self, attempt):
+    def __call__(self, retry_state):
         # always retry if no exception was raised
-        if not attempt.failed:
+        if not retry_state.outcome.failed:
             return True
-        return self.predicate(attempt.exception())
+        return self.predicate(retry_state.outcome.exception())
 
 
 class retry_if_result(retry_base):
@@ -93,9 +102,11 @@ class retry_if_result(retry_base):
     def __init__(self, predicate):
         self.predicate = predicate
 
-    def __call__(self, attempt):
-        if not attempt.failed:
-            return self.predicate(attempt.result())
+    def __call__(self, retry_state):
+        if not retry_state.outcome.failed:
+            return self.predicate(retry_state.outcome.result())
+        else:
+            return False
 
 
 class retry_if_not_result(retry_base):
@@ -104,9 +115,61 @@ class retry_if_not_result(retry_base):
     def __init__(self, predicate):
         self.predicate = predicate
 
-    def __call__(self, attempt):
-        if not attempt.failed:
-            return not self.predicate(attempt.result())
+    def __call__(self, retry_state):
+        if not retry_state.outcome.failed:
+            return not self.predicate(retry_state.outcome.result())
+        else:
+            return False
+
+
+class retry_if_exception_message(retry_if_exception):
+    """Retries if an exception message equals or matches."""
+
+    def __init__(self, message=None, match=None):
+        if message and match:
+            raise TypeError(
+                "{}() takes either 'message' or 'match', not both".format(
+                    self.__class__.__name__
+                )
+            )
+
+        # set predicate
+        if message:
+
+            def message_fnc(exception):
+                return message == str(exception)
+
+            predicate = message_fnc
+        elif match:
+            prog = re.compile(match)
+
+            def match_fnc(exception):
+                return prog.match(str(exception))
+
+            predicate = match_fnc
+        else:
+            raise TypeError(
+                "{}() missing 1 required argument 'message' or 'match'".format(
+                    self.__class__.__name__
+                )
+            )
+
+        super(retry_if_exception_message, self).__init__(predicate)
+
+
+class retry_if_not_exception_message(retry_if_exception_message):
+    """Retries until an exception message equals or matches."""
+
+    def __init__(self, *args, **kwargs):
+        super(retry_if_not_exception_message, self).__init__(*args, **kwargs)
+        # invert predicate
+        if_predicate = self.predicate
+        self.predicate = lambda *args_, **kwargs_: not if_predicate(*args_, **kwargs_)
+
+    def __call__(self, retry_state):
+        if not retry_state.outcome.failed:
+            return True
+        return self.predicate(retry_state.outcome.exception())
 
 
 class retry_any(retry_base):
@@ -115,8 +178,8 @@ class retry_any(retry_base):
     def __init__(self, *retries):
         self.retries = retries
 
-    def __call__(self, attempt):
-        return any(map(lambda x: x(attempt), self.retries))
+    def __call__(self, retry_state):
+        return any(r(retry_state) for r in self.retries)
 
 
 class retry_all(retry_base):
@@ -125,5 +188,5 @@ class retry_all(retry_base):
     def __init__(self, *retries):
         self.retries = retries
 
-    def __call__(self, attempt):
-        return all(map(lambda x: x(attempt), self.retries))
+    def __call__(self, retry_state):
+        return all(r(retry_state) for r in self.retries)
