@@ -24,25 +24,21 @@ from tornado import gen
 
 
 class TornadoRetrying(BaseRetrying):
-
-    def __init__(self,
-                 sleep=gen.sleep,
-                 **kwargs):
+    def __init__(self, sleep=gen.sleep, **kwargs):
         super(TornadoRetrying, self).__init__(**kwargs)
         self.sleep = sleep
 
     @gen.coroutine
-    def call(self, fn, *args, **kwargs):
+    def __call__(self, fn, *args, **kwargs):
         self.begin(fn)
 
-        retry_state = RetryCallState(
-            retry_object=self, fn=fn, args=args, kwargs=kwargs)
+        retry_state = RetryCallState(retry_object=self, fn=fn, args=args, kwargs=kwargs)
         while True:
             do = self.iter(retry_state=retry_state)
             if isinstance(do, DoAttempt):
                 try:
                     result = yield fn(*args, **kwargs)
-                except BaseException:
+                except BaseException:  # noqa: B902
                     retry_state.set_exception(sys.exc_info())
                 else:
                     retry_state.set_result(result)
